@@ -1,7 +1,7 @@
 package Crypt::Password::Util;
 
-our $DATE = '2015-01-25'; # DATE
-our $VERSION = '0.10'; # VERSION
+our $DATE = '2015-01-31'; # DATE
+our $VERSION = '0.11'; # VERSION
 
 use 5.010001;
 use strict;
@@ -17,40 +17,71 @@ my $hexd = qr![0-9a-f]!;
 our %CRYPT_TYPES = (
     'MD5-CRYPT' => {
         summary => 'A baroque passphrase scheme based on MD5, designed by Poul-Henning Kamp and originally implemented in FreeBSD',
-        re => qr/\A \$ (?:apr)?1 \$ $b64d {0,8} \$ $b64d {22} \z/x,
+        re => qr/\A
+                 (?P<header>\$ (?:apr)?1 \$)
+                 (?P<salt>$b64d {0,8}) \$
+                 (?P<hash>$b64d {22}) \z/x,
         re_summary => '$1$ or $apr1$ header',
+        link => 'http://static.usenix.org/event/usenix99/provos/provos_html/node10.html',
     },
     CRYPT => {
         summary => 'Traditional DES crypt',
-        re => qr/\A .. $b64d {11} \z/x,
+        re => qr/\A
+                 (?P<salt>..)
+                 (?P<hash>$b64d {11}) \z/x,
         re_summary => '11 digit base64 characters',
+        link => 'http://perldoc.perl.org/functions/crypt.html',
     },
     SSHA256 => {
         summary => 'Salted SHA256, supported by glibc 2.7+',
-        re => qr/\A \$ 5 \$ $b64d {0,16} \$ $b64d {43} \z/x,
+        re => qr/\A
+                 (?P<header>\$ 5 \$)
+                 (?P<salt>$b64d {0,16}) \$
+                 (?P<hash>$b64d {43}) \z/x,
         re_summary => '$5$ header',
+        link => 'http://en.wikipedia.org/wiki/SHA-2',
     },
     SSHA512 => {
         summary => 'Salted SHA512, supported by glibc 2.7+',
-        re => qr/\A \$ 6 \$ $b64d {0,16} \$ $b64d {86} \z/x,
+        re => qr/\A
+                 (?P<header>\$ 6 \$)
+                 (?P<salt>$b64d {0,16}) \$
+                 (?P<hash>$b64d {86}) \z/x,
         re_summary => '$6$ header',
+        link => 'http://en.wikipedia.org/wiki/SHA-2',
     },
     BCRYPT => {
         summary => 'Passphrase scheme based on Blowfish, designed by Niels Provos and David Mazieres for OpenBSD',
-        re => qr/\A \$ 2a? \$ \d+ \$ $b64d {53} \z/x,
-        re_summary => '$2$ or $2a$header followed by 22 base64-digits salt and 31 digits hash',
+        re => qr/\A
+                 (?P<header>\$ 2a? \$)
+                 (?P<cost>\d+) \$
+                 (?P<salt>$b64d {22})
+                 (?P<hash>$b64d {31}) \z/x,
+        re_summary => '$2$ or $2a$ header followed by cost, followed by 22 base64-digits salt and 31 digits hash',
+        link => 'https://www.usenix.org/legacy/event/usenix99/provos/provos_html/',
     },
     'PLAIN-MD5' => {
         summary => 'Unsalted MD5 hash, popular with PHP web applications',
-        re => qr/\A $hexd {32} \z/x,
+        re => qr/\A (?P<hash>$hexd {32}) \z/x,
         re_summary => '32 digits of hex characters',
+        link => 'http://en.wikipedia.org/wiki/MD5',
     },
 );
 
 sub crypt_type {
     my $crypt = shift;
+    my $detail = shift;
+
     for my $type (keys %CRYPT_TYPES) {
-        return $type if $crypt =~ $CRYPT_TYPES{$type}{re};
+        if ($crypt =~ $CRYPT_TYPES{$type}{re}) {
+            if ($detail) {
+                my $res = {%+};
+                $res->{type} = $type;
+                return $res;
+            } else {
+                return $type;
+            }
+        }
     }
     return undef;
 }
@@ -96,7 +127,7 @@ Crypt::Password::Util - Crypt password utilities
 
 =head1 VERSION
 
-This document describes version 0.10 of Crypt::Password::Util (from Perl distribution Crypt-Password-Util), released on 2015-01-25.
+This document describes version 0.11 of Crypt::Password::Util (from Perl distribution Crypt-Password-Util), released on 2015-01-31.
 
 =head1 SYNOPSIS
 
@@ -108,16 +139,29 @@ This document describes version 0.10 of Crypt::Password::Util (from Perl distrib
  say crypt_type('$5$4DdvgCFk$...');                  # SSHA256
  say crypt_type('$6$4DdvgCFk$...');                  # SSHA512
  say crypt_type('1a1dc91c907325c69271ddf0c944bc72'); # PLAIN-MD5
+ say crypt_type('$2a$08$TTSynMjJTrXiv3qEZFyM1.H9tjv71i57p2r63QEJe/2p0p/m1GIy2'); # BCRYPT
  say crypt_type('foo');                              # undef
 
+ # return detailed information
+ say crypt_type('$1$$oXYGukVGYa16SN.Pw5vNt/', 1);
+ # => {}
+
  say looks_like_crypt('62F4a6/89.12z');   # 1
  say looks_like_crypt('foo');             # 0
 
  say crypt('pass'); # automatically choose the appropriate type and salt
 
+=head1 DESCRIPTION
+
+Crypt::Password::Util facilitates the generation and recognition of unix
+passwords as found in /etc/shadow on Unix/Linux systems and /etc/master.passwd
+on BSD systems. When using crypt(), it is possible several methods will be
+attempted before returning a result. This is done to insure that your system
+supports the selected hash type.
+
 =head1 FUNCTIONS
 
-=head2 crypt_type($str) => STR
+=head2 crypt_type($str[, $detail]) => str|hash
 
 Return crypt type, or undef if C<$str> does not look like a crypted password.
 Currently known types:
@@ -128,7 +172,9 @@ Currently known types:
 
 Passphrase scheme based on Blowfish, designed by Niels Provos and David Mazieres for OpenBSD.
 
-Recognized by: $2$ or $2a$header followed by 22 base64-digits salt and 31 digits hash.
+Recognized by: $2$ or $2a$ header followed by cost, followed by 22 base64-digits salt and 31 digits hash.
+
+More info: L<https://www.usenix.org/legacy/event/usenix99/provos/provos_html/>
 
 =item * CRYPT
 
@@ -136,32 +182,45 @@ Traditional DES crypt.
 
 Recognized by: 11 digit base64 characters.
 
+More info: L<http://perldoc.perl.org/functions/crypt.html>
+
 =item * MD5-CRYPT
 
 A baroque passphrase scheme based on MD5, designed by Poul-Henning Kamp and originally implemented in FreeBSD.
 
 Recognized by: $1$ or $apr1$ header.
 
+More info: L<http://static.usenix.org/event/usenix99/provos/provos_html/node10.html>
+
 =item * PLAIN-MD5
 
 Unsalted MD5 hash, popular with PHP web applications.
 
 Recognized by: 32 digits of hex characters.
 
+More info: L<http://en.wikipedia.org/wiki/MD5>
+
 =item * SSHA256
 
 Salted SHA256, supported by glibc 2.7+.
 
 Recognized by: $5$ header.
 
+More info: L<http://en.wikipedia.org/wiki/SHA-2>
+
 =item * SSHA512
 
 Salted SHA512, supported by glibc 2.7+.
 
 Recognized by: $6$ header.
 
+More info: L<http://en.wikipedia.org/wiki/SHA-2>
+
 =back
 
+If C<$detail> is set to true, will return a hashref of information instead. This
+include C<type>, as well as the parsed header, salt, etc.
+
 =head2 looks_like_crypt($str) => BOOL
 
 Return true if C<$str> looks like a crypted password.
